'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var fs = require('fs');
var path = require('path');

var autoprefixer = require('autoprefixer');
var cssnano = require('cssnano');
var less = require('less');
var NpmImportPlugin = require('less-plugin-npm-import');
var postcss = require('postcss');
var sass = require('node-sass');

var Core = require('css-modules-loader-core');
var camelCaser = require('./camelCaser').default;

var core = new Core([Core.localByDefault, Core.extractImports, camelCaser, Core.scope]);

/**
 * replace the tilde-based url with the path to the node module
 *
 * @param {string} url
 * @returns {string}
 */
var resolveUrlPath = function resolveUrlPath(url) {
  if (url[0] !== '~') {
    return url;
  }

  return path.resolve(path.dirname(require.main.filename), 'node_modules', url.slice(1));
};

/**
 * save the transformed file to target
 *
 * @param {string} hash
 * @param {string} target
 * @returns {function({exportTokens: object, injectableSource: string}): Object}
 */
var saveFile = function saveFile(hash, target) {
  return function (_ref) {
    var exportTokens = _ref.exportTokens,
        injectableSource = _ref.injectableSource;

    // provide the compiled CSS, the id to use in the style tag, and the selector map
    var fileValue = {
      css: injectableSource,
      id: hash + '_Scoped_Styles',
      selectors: exportTokens
    };

    // clean the string to mean the rapid7 ESLint config rules
    var bufferString = JSON.stringify(fileValue).replace(/'/g, '\\\'').replace(/"/g, '\'').replace(/,/g, ', ');

    // make a Buffer of an exportable file
    var buffer = new Buffer('module.exports = ' + bufferString + ';\n');

    // write the file to the target
    fs.writeFileSync(target, buffer);

    return fileValue;
  };
};

/**
 * load the prefixed CSS and convert via CSS Modules, then create a file with all metadata provided
 *
 * @param {string} hash
 * @param {string} target
 * @returns {function({css: string}): Promise}
 */
var applyCssModules = function applyCssModules(hash, target) {
  return function (_ref2) {
    var css = _ref2.css;

    return core.load(css, hash).catch(function (error) {
      /* eslint-disable no-console */
      console.error('There was an error converting your file via CSS Modules.');
      /* eslint-enable */

      throw new Error(error);
    }).then(saveFile(hash, target));
  };
};

/**
 * get the CSS buffer from rendering the file
 *
 * @param {string} source
 * @param {Object} renderOptions
 * @returns {function(resolve: function): void}
 */
var getRenderedCss = function getRenderedCss(source, renderOptions) {
  return function (resolve) {
    switch (path.extname(source).slice(1)) {
      case 'scss':
        var scssOptions = Object.assign({}, renderOptions, {
          file: source,
          importer: function importer(url) {
            return {
              file: resolveUrlPath(url)
            };
          }
        });

        // if the file is SCSS, use node-sass to compile into a Buffer
        var renderedFile = sass.renderSync(scssOptions);

        resolve(renderedFile.css);

        break;

      case 'less':
        // if the file is LESS, convert to string and render
        var fileString = fs.readFileSync(source, 'utf8');
        var lessOptions = Object.assign({}, renderOptions, {
          filename: source,
          plugins: [].concat(_toConsumableArray(renderOptions.plugins || []), [new NpmImportPlugin({
            prefix: '~'
          })])
        });

        less.render(fileString, lessOptions).then(function (_ref3) {
          var css = _ref3.css;

          resolve(css);
        });

        break;

      default:
        // otherwise, assume the file is plain CSS and read the file as a Buffer
        resolve(fs.readFileSync(source, 'utf8'));
    }
  };
};

/**
 * get the prefixed css and apply the CSS modules
 *
 * @param {string} hash
 * @param {string} target
 * @param {boolean} minify
 * @returns {function(cssBuffer: Buffer): Promise}
 */
var getPrefixedCss = function getPrefixedCss(hash, target, minify) {
  return function (cssBuffer) {
    // convert the Buffer to a string and prefix via postcss
    var string = cssBuffer.toString('utf8');

    var prefixerPlugins = [autoprefixer({
      remove: false
    })];

    if (minify) {
      prefixerPlugins = [].concat(_toConsumableArray(prefixerPlugins), [cssnano]);
    }

    var prefixer = postcss(prefixerPlugins);

    return prefixer.process(string).catch(function (error) {
      /* eslint-disable no-console */
      console.error('There was an error prefixing your css.');
      /* eslint-enable */

      throw new Error(error);
    }).then(applyCssModules(hash, target));
  };
};

/**
 * generate the JS file with the appropriate prefixed
 * styles rendered and modularized
 *
 * @param {string} hash
 * @param {boolean} minify
 * @param {Object} renderOptions
 * @param {string} source
 * @param {string} target
 * @returns {Promise}
 */
var generateTransformedStylesFile = function generateTransformedStylesFile(hash, minify, renderOptions, source, target) {
  return new Promise(getRenderedCss(source, renderOptions)).catch(function (error) {
    /* eslint-disable no-console */
    console.error('There was a problem generating the CSS from your source file.');
    /* eslint-enable */

    throw new Error(error);
  }).then(getPrefixedCss(hash, target, minify));
};

module.exports = {
  applyCssModules: applyCssModules,
  generateTransformedStylesFile: generateTransformedStylesFile,
  getPrefixedCss: getPrefixedCss,
  getRenderedCss: getRenderedCss,
  resolveUrlPath: resolveUrlPath,
  saveFile: saveFile
};